function out = permgolden(n,k,norep)
% PERMGOLDEN creates a permutation vector    
%   OUT = PERMGOLDEN(N) creates the vector of size N. This function assumes
%   a cyclic vector. In other words, element N+1 should correspond to
%   element 1.
%
%   OUT = PERMGOLDEN(N,K) creates the vector of size K using golden cut
%   permutation. All elements in OUT is <= N.
%    
%   OUT = PERMGOLDEN(N,K,NOREP) does not repeat elements in OUT until all
%   elements are collected. This is for NOREP = 1.  The output of
%   PERMGOLDEN(N,N,1) is equivalent to PERMGOLDEN(N).
%
%   Note this function is optimal if N is a Fibonacci number.
%     [1 2 3 5 8 13 21 34 55 89 144 233 377 610 987 ...]
%
%   See also ROUNDFIB and FLOORFIB.
%
% (c) Joseph Y Cheng (jycheng@mrsrl.stanford.edu) 2012
%
 
% SVN info: 
%     Date:     $Date: 2013-12-09 17:14:59 -0800 (Mon, 09 Dec 2013) $
%     Revision: $Revision: 1389 $
%     Author:   $Author: jycheng $
%     Id:       $Id: permgolden.m 1389 2013-12-10 01:14:59Z jycheng $
        
    if nargin < 2
        k = n;
    end
    if nargin < 3
        norep = 1;
    end
    
    %% golden ratio
    gr = (sqrt(5)-1)/2;
    inc_gr = gr*n;
    %% delta between each element
    dv = 1;
    
    %% Current value
    V = 0; 
    N = 1;
    out = zeros(1,k);    
    %% determine if current element is already in the 
    %% resulting permutation array
    exist = zeros(1,n);
    
    while N<=k
        
        %% convert to indeces
        I = round(V/dv)+1;
        if I>n
            I=1;
        end
        
        %% add to output
        if ~exist(I) || ~norep
            out(N) = I;
            exist(I) = 1;
            N = N+1;
        end
        
        %% increment by golden ratio
        V = V + inc_gr;
        while V > n
            V = V - n;
        end
        
        if norep && sum(exist(:))>=n
            exist = zeros(1,n);
        end
        
    end
    
    
    