function [yout,zout,mask] = vo_capr(nsize,ncal,nvanes,numph,param)
% VO_CAPR view ordering for CAPR
%   [YO,ZO] = VO_CAPR(FOV,NSIZE,NCAL,NVANES,NPH,[PARAM])
%   NSIZE -- [int]   [ny nz] size of acq matrix
%   NCAL  -- [int]   size of calibration area
%   NUMPH -- [int]   num phases to output
%   PARAM -- [CELL]  optional argument specifying different features:
%      'dbprint'   -- [int]    if 1, verbose print outs
%                                (default = 0)
%      'dbplot'    -- [int]    if>0, plot figure with figure
%                              number specified by the argument
%                                (default = 0)
%
%   Output:
%    YO -- [int] [nviews] ky ordering 
%    ZO -- [int] [nviews] kz ordering
%
%   Some conventions with CAPR (CArtesian Projection Reconstruction):
%     * 2 fold undersampling of vanes
%     * ~32 vanes
%     * corner cut
%
%   References:
%    [1] Haider CR, Hu HH, Campeau NG, Houston J, Riederer SJ. 3D high
%        temporal and spatial resolution contrast-enhanced MR angiography of
%        the whole brain. Magn Reson Med 2008;60:749-760.
%    [2] Madhuranthakam AJ, Hu HH, Barger AV, Haider CR, Kruger DG,
%        Glockner JF, Houston J 3rd, Riederer SJ. Undersampled elliptical
%        centric view-order for improved spatial resolution in
%        contrast-enhanced MR angiography. Magn Reson Med 2006;55:50--58.
%
%   See also VO_SEQ, VO_ELLIPCENT, VO_RADIAL, and VO_VDRAD.
%   See also VO_DRAWORDER and VO_DRAWORDER_VID.
%
%   Depends on DBDISP, VO_CORNERCUT, SFIGURE, VO_ELLIPCENT.
%
% (c) Joseph Y Cheng (jycheng@mrsrl.stanford.edu) 2013
%
    
% SVN info: 
%     Date:     $Date: 2014-10-16 11:18:02 -0700 (Thu, 16 Oct 2014) $
%     Revision: $Revision: 1442 $
%     Author:   $Author: jycheng $
%     Id:       $Id: vo_capr.m 1442 2014-10-16 18:18:02Z jycheng $ 
        
    if nargin<4
        numph = nvanes/2;
    end
    if nargin<5
        param = [];
    end
    
    if mod(nvanes,numph)~=0 
        dbdisp(sprintf(['ERROR! nvanes (%d) must be divisible by numph ' ...
                        '(%d)'], nvanes, numph));
        return;
    end
    nvanes_log2 = log2(nvanes);
    numph_log2  = log2(numph);
    if nvanes_log2~=floor(nvanes_log2) || ...
            numph_log2~=floor(numph_log2)
        dbdisp(sprintf(['ERROR! nvanes (%d) and numph (%d) must be a ' ...
                        'power of 2.'],nvanes,numph));
        return
    end
    
    %% Extract parameters
    p = parseparam(param);
    DBPLOT    = p.dbplot;
    DBPRINT   = p.dbprint;
    
    %% Init output
    yout = []; zout = [];
    
    %% num of points
    ny = nsize(1);
    nz = nsize(2);
    
    %% Find initial ordering (based on seq)
    yi = repmat((1:ny),[nz 1]);
    zi = repmat((1:nz).',[1 ny]);
    yi = yi(:); zi = zi(:);
    
    %% Convert ordering to k-space coordinate points
    ky = -1+2*(yi-.5)/ny;
    if nz>1
        kz = -1+2*(zi-.5)/nz;
    else
        kz = 0*zi;
    end

    %if cornercut
    %% CAPR always has corner cut
        ind = vo_cornercut(ky,kz,nsize);
        yi = yi(ind); zi = zi(ind);
        ky = ky(ind); kz = kz(ind);
        clear ind;        
    %end
   
    ntot = length(yi(:));
    
    %% convert to complex space for easier calc
    %kact = rs(1)*ky  + i*rs(2)*kz;
    kact = ky + i*kz;
    
    %% used for sorting
    vact = [angle(kact(:)),abs(kact(:)),(1:length(kact(:))).'];
    %% find calib/center region (always corner cut cal)
    if ny>nz
        r0 = ncal/ny;
    else
        r0 = ncal/nz;
    end
    numcalib = sum(vact(:,2)<=r0);
    vact = sortrows(vact,2);
    vc = vact(1:numcalib,:);
    yc = yi(vc(:,end));
    zc = zi(vc(:,end));
    
    vo = vact((numcalib+1):end,:);
    vo = sortrows(vo,[1 2]);
    
    nvanefull = 2*nvanes;
    lvane  = round(size(vo,1)/nvanefull);    
    lvane0 = lvane + (size(vo,1) - lvane*nvanefull);
    
    %% divide up outer views into vanes
    ytab = [];
    ztab = [];    
    for P=1:numph
        yp = []; zp = [];
        for N=P:numph:nvanes
            if N<=nvanes/2
                indn = 2*(N-1)*lvane+[1:lvane];
            else
                indn = (2*(N-1)-1)*lvane+[1:lvane];
            end
            yp = [yp; yi(vo(indn,end))];
            zp = [zp; zi(vo(indn,end))];
        end
        ytab{P} = yp;
        ztab{P} = zp;
    end

    if DBPLOT
        sfigure(DBPLOT);hold off;
        for P=1:numph
            if mod(P,2)
                plot(ytab{P},ztab{P},'k.');
            else
                plot(ytab{P},ztab{P},'b.');
            end
            hold on
        end
    end
    
    %% Finalize:
    %%  - interleave center region 
    %%  - interleave phases
    %%  - re-order using elliptical centric
    yout = [];
    zout = [];
    [yc,zc] = vo_ellipcent(yc,zc);
    for P=bitrevorder(1:numph)
        [ytmp,ztmp] = vo_ellipcent(ytab{P},ztab{P});
        yout = [yout; yc(:); ytmp(:)];
        zout = [zout; zc(:); ztmp(:)];
    end
    
    if nargout>2
        mask = zeros(nsize);
        for N=1:length(yout)
            mask(yout(N),zout(N)) = 1;
        end
    end
    
    return;
    
function p = parseparam(paramcell)
    
    if mod(size(paramcell),2) ~= 0
        dbdisp('Invalid parameters!');
        p = -1;
        return;
    end
    
    %% default
    p = [];
    p.dbprint   = 0;
    p.dbplot    = 0;
    
    %% look through user defined params
    for N=1:(size(paramcell,2)/2)
        pname = paramcell{2*N-1};
        parg  = paramcell{2*N};
        switch pname
         case 'dbplot'
          p.dbplot = parg;
         case 'dbprint'
          p.dbprint = parg;
         case 'demo'
          p.demo = parg;
         otherwise
          dbdisp(sprintf('Ignoring invalid parameter %s.',pname));
        end
    end