function [yout,zout] = vo_radial(yin,zin,tl,mode)
% VO_RADIAL generates radial view-ordering
%   [YO,ZO] = VO_RADIAL(YI,ZI,TL,MODE) generates radial view-ordering
%   based on a given view-ordering YIN and ZIN. TL specifies the train
%   length of each radial spoke. The MODE determines the ordering of
%   each radial spoke:
%     MODE -- [int] 0, center to out order
%                   1, out to center order
%                   2, center to out and back
%
%   Output:
%     YO -- [int] [nviews] ky ordering 
%     ZO -- [int] [nviews] kz ordering
%
%   Suggested ref: 
%     [1] Busse, RF, Brau, ACS, Vu A, Michelich CR, Bayram E, Kijowski R, 
%         Reeder SB, Rowley HA. Effects of refocusing flip angle modulation
%         and view ordering in 3D fast spin echo 2008. Magn Reson Med, 60:
%         640–649.
%
%     [2] Winkelmann S, Schaeffter T, Koehler T, Eggers H, Doessel O. An
%         optimal radial profile order based on the Golden Ratio for
%         time-resolved MRI. IEEE Trans Med Imaging 2007; 26:68–76.
%
%   
%   See also VO_SEQ, VO_ELLIPCENT, VO_CAPR, and VO_VDRAD.
%   See also VO_DRAWORDER and VO_DRAWORDER_VID.
%   
%   Depends on PERMGOLDEN.
%
% (c) Joseph Y Cheng (jycheng@stanford.edu) 2013.
%
    
% SVN info: 
%     Date:     $Date: 2014-10-16 11:18:02 -0700 (Thu, 16 Oct 2014) $
%     Revision: $Revision: 1442 $
%     Author:   $Author: jycheng $
%     Id:       $Id: vo_radial.m 1442 2014-10-16 18:18:02Z jycheng $ 
    
    if nargin<3
        tl = 24;
    end
    if nargin<4
        mode = 0;
    end
    
    y = yin - min(yin(:)); 
    z = zin - min(zin(:));
   
    ymax = max(y(:));
    ymid = ymax/2;    
    zmax = max(z(:));
    zmid = zmax/2;
    
    ky = (y-ymid)/ymax;
    kz = (z-zmid)/zmax;    
    k = ky+i*kz;
    
    v = [angle(k(:)),abs(k(:)),(1:length(k)).'];    
    v = sortrows(v,[1 2]);
   
    npe   = length(yin(:));
    ntl   = floor(npe/tl);
    extra = (npe-ntl*tl);
    yout = []; zout = [];        
    tl1 = round(tl/2);
    %ga = 111.246/180*pi;
                    
    for I=permgolden(ntl)
        %% index for spoke
        indi = (1:tl)+(I-1)*tl;
        indi = indi(find(indi<npe));
        
        %% extract spoke
        vi = v(indi,:);
            
        switch mode
         case 0
          %% Center to outer
          vi = sortrows(vi,2);
          yout = [yout; yin(vi(:,end)); yin(vi(:,end))];
          zout = [zout; zin(vi(:,end)); zin(vi(:,end))];                 
                 
         case 1
          %% Outer to center
          vi = sortrows(vi,-2);
          yout = [yout; yin(vi(:,end)); yin(vi(:,end))];
          zout = [zout; zin(vi(:,end)); zin(vi(:,end))];
          
         case 2
          %% Center to outer and BACK!
          if length(vi)>tl1
               vi1 = sortrows(vi(1:tl1,:),2);
               vi2 = sortrows(vi((tl1+1):end,:),-2);
               yout = [yout; yin(vi1(:,end)); yin(vi2(:,end))];
               zout = [zout; zin(vi1(:,end)); zin(vi2(:,end))];
          else
              vi1 = sortrows(vi(1:end,:),2);
              yout = [yout; yin(vi1(:,end))];
              zout = [zout; zin(vi1(:,end))];
          end
        end
    end
    
    