function [yout,zout,pout,mask] = vo_vdrad(fov,nsize,acc,ncal,tl,numph,param)
% VO_VDRAD accelerated radial view ordering
%   [YO,ZO] = VO_VDRAD(FOV,SIZE,ACC,NCAL,TL,NUMPH,[PARAM]) generates a
%   Variable-Density sampling and Radial view-ordering (VDRad). The 
%   arguments of the function include:
%     FOV   -- [mm]    [FOVy FOVz] 
%     NSIZE -- [int]   [ny nz] size of acq matrix
%     ACC   -- [float] [ay az] acceleration factor
%     NCAL  -- [int]   size of calibration area
%     TL    -- [int]   train length of each spoke
%     NUMPH -- [int]   num phases to output -- one phase is defined as when the
%                      calibration area is fully sampled and the desired
%                      acceleration is achieved.  
%                       (default = 1)
%     PARAM -- [CELL]  optional argument specifying different features:
%       'type'      -- [string] shape of each ring.
%                               options: 'ellipse','square','triangle'
%                                (default = 'ellipse')
%       'mode'      -- [int]    ordering of each spoke
%                               options: 1-out to center, 2-center out, back
%                               otherwise: center to out
%                                (default = 0)
%       'dofib'     -- [int]    round the smallest dofib rings to the 
%                               nearest Fibonacci number. This covers the 
%                               rings as best as possible for 1 temporal phase
%                               at the cost of the desired acc factor.
%                                (default = 1)
%       'corner'    -- [int]    if 1, corner cut k-space 
%                                (default = 1)
%       'cornercal' -- [int]    if 1, corner cut k-space in calib region 
%                                (default = 1)
%       'spiral'    -- [int]    adds a spiral twist to each radial spoke
%                                (default = 10)    
%       'dbprint'   -- [int]    if 1, verbose print outs
%                                (default = 0)
%       'demo'      -- [int]    if>0, plot figure with figure
%                               number specified by the argument
%                                (default = 0)
%       'calnex'    -- [int]    NEX factor for calibration region for 1 phase
%                                (default = 1)
%       'ncalmin'   -- [int]    the calibration region size may have to be 
%                               adjusted for a given matrix and acceleration.
%                               this is the min size of calib region
%                                (default = 10)
%   Output:
%     YO -- [int] [nviews] ky ordering 
%     ZO -- [int] [nviews] kz ordering
%
%   Examples:
%     [yo,zo] = vo_vdrad([300 300],[256 256],[4 2],...
%                        24,24,1,{'type','square'});
%     [yo,zo] = vo_vdrad([300 200],[256 256],[4 2],...
%                        24,24,1,{'dofib','4'});
%
%   References:
%     [1] Cheng JY, Zhang T, Ruangwattanapaisarn N, Alley MT, Uecker M,
%         Pauly JM, Lustig M, Vasanawala SS. Free-breathing pediatric
%         imaging with nonrigid motion correction and acceleratedion. J Magn
%         Reson Imaging 2014. Accepted Oct 6, 2014.
%     [2] Cheng JY, Zhang T, Alley MT, Lustig M, Vasanawala SS, Pauly
%         JM. Variable-density radial view-ordering and sampling for
%         time-optimized 3D Cartesian imaging. In Proceedings of the ISMRM
%         Workshop on Data Sampling and Image Reconstruction, Salt Lake
%         City, Utah, USA, 2013.
%     [3] Winkelmann S, Schaeffter T, Koehler T, Eggers H, Doessel O. An
%         optimal radial profile order based on the Golden Ratio for
%         time-resolved MRI. IEEE Trans Med Imaging 2007; 26:68–76.
%     [4] Busse, RF, Brau, ACS, Vu A, Michelich CR, Bayram E, Kijowski R, 
%         Reeder SB, Rowley HA. Effects of refocusing flip angle modulation
%         and view ordering in 3D fast spin echo 2008. Magn Reson Med, 60:
%         640–649.

%
%   See also VO_SEQ, VO_ELLIPCENT, VO_RADIAL, and VO_CAPR.
%   See also VO_DRAWORDER and VO_DRAWORDER_VID.
%
%   Depends on FLOORFIB, DBDISP, and VO_CORNERCUT.
%
% (c) Joseph Y Cheng (jycheng@mrsrl.stanford.edu) 2013-2014
%
    
% Log: 
%   - Updated (2013 Feb 28 jycheng).
%   - Added 'demo' feature (2013 Jun 7 jycheng)
%   - Matching variable names with paper (2013 Jun 10 jycheng)    
%   - Clean up some code. Added hack to consider if prod(acc) is close to 1
%     (2014 Oct 14 jycheng)
   
% SVN info:
%     Date:     $Date: 2014-10-16 11:18:02 -0700 (Thu, 16 Oct 2014) $
%     Revision: $Revision: 1442 $
%     Author:   $Author: jycheng $
%     Id:       $Id: vo_vdrad.m 1442 2014-10-16 18:18:02Z jycheng $
    
    if nargin<6
        numph = 1;
    end
    if nargin<7
        param = [];
    end

    % Extract parameters
    p = parseparam(param);
    shape     = p.type;
    fib       = p.dofib;
    cornercut = p.corner;
    ccutcal   = p.cornercal;
    spiral    = p.spiral;
    mode      = p.mode;
    calnex    = p.calnex;
    ncalmin   = p.ncalmin;
    vdrate    = p.vdrate;
    acctol    = p.acctol;
    DBPLOT    = p.dbplot;
    DBPRINT   = p.dbprint;
    DEMO      = p.demo;
    
    % Check some params
    if fib<0
        fib = 0;
    elseif fib>tl-1
        fib = tl-1;
    end
    if spiral<0
        spiral = 0;
    end
    if calnex<1
        calnex = 1;
    end
    if DEMO<0
        DEMO = 0;
    end    
    
    % Init output
    yout = []; zout = [];
    
    % FOV [mm]
    fovy = fov(1);
    fovz = fov(2);
    
    % num of points
    ny = nsize(1);
    nz = nsize(2);
    
    % acceleration
    ry = (acc(1));
    rz = (acc(2));
    if nz==1
        rz = 1;
    end
    if ry*rz<1
        dbdisp(sprintf('ERROR! Invalid acceleration: (%g,%g)',ry,rz));
        return;
    end
    if ry*rz < 1.01*1.01
        % HACK: if ry*rz too low, the sampling generation takes forever to
        %       try to get it close to that acceleration
        dbdisp(sprintf('WARNING: desired acc too low (%g,%g)!!',...
                       ry,rz));
        ry = ry*1.01;
        rz = rz*1.01;
        dbdisp(sprintf('  adjusting acc factors to (%g,%g).',...
                       acc(1),acc(2)));
        dbdisp('');
        dbdisp('');
    end

    
    %% Acceleration setup    
    % Find initial ordering (based on seq)
    yi = repmat((1:ny),[nz 1]);
    zi = repmat((1:nz).',[1 ny]);
    yi = yi(:); zi = zi(:);
    
    % Convert ordering to k-space coordinate points normalized from (-1,1]
    ky = -1+2*(yi-.5)/ny;
    if nz>1
        kz = -1+2*(zi-.5)/nz;
    else
        kz = 0*zi;
    end
    
    % Corner cut (based on Miki's code)
    if cornercut
        ind = vo_cornercut(ky, kz, nsize);

        yi = yi(ind); zi = zi(ind);
        ky = ky(ind); kz = kz(ind);
        
        clear ind;
    end
    
    ntot = length(yi(:));
    
    % Find calib area (based on Miki's code).    
    %  Make sure same resolution in both directions.
    %  Adjust scaling to get same extent in k-space in bother directions
    % resolution [mm] (1/fovmax)
    res = fov./nsize; 
    % resolution ratio
    rs = [1 1];
    % find largest res
    if res(1)>res(2)
        rs(1) = res(2)/res(1);
    else
        rs(2) = res(1)/res(2);
    end

    % Adjust for acceleration
    rry  = 1; rrz = 1;
    if rz>ry
        rry = (ry/rz);
    else
        rrz = (rz/ry);
    end
    
    
    %% Calculate corresponding k-space radius (k_r)
    %         and the k-space angle (k_theta)
    
    % Convert to complex space for easier calc
    kadj = ky*rry*rs(1) + i*kz*rrz*rs(2); 
    kact = rs(1)*ky  + i*rs(2)*kz;
    
    % k-space radius (k_r)
    switch shape
     case 1 % square
      va = max(abs(real(kadj(:))),abs(imag(kadj(:))));
     case 2 % triangle (separable)
      va = abs(real(kadj(:)))+abs(imag(kadj(:)));
     otherwise % ellipse
      va = abs(kadj(:));
    end
    % k-space angle (k_theta)
    vp = angle((kadj(:)).*exp(i*spiral*va));

    if DBPLOT
        sampa = zeros(nsize);
        sampp = zeros(nsize);
        for N=1:length(kadj)
            sampa(yi(N),zi(N)) = va(N);
            sampp(yi(N),zi(N)) = vp(N);
        end
        maxa = max(va(:));
        mina = min(va(:));
        sfigure(DBPLOT);
        subplot(121),imshow(sampa,[mina+0.1 maxa]),title('radius');
        colorbar;
        subplot(122),imshow(sampp,[]),title('angle'),colorbar;
        drawnow;
    end
    
    % [phase, mag, index]
    v = [vp,va,(1:length(kadj(:))).']; % adjusted
    vact = [angle(kact(:)),abs(kact(:)),(1:length(kact(:))).']; % actual
    
    ry_n = ry;
    rz_n = rz;
    calnex_iter = 0;
    ncal_n = ncal;    
    lring = tl;
    
    % Loop through to adjust for calnex
    while 1        
        
        % Adjust calibration region size to fit desired acceleration
        numcalib = ntot*2;
        ncal_i = ncal_n;
        while ry_n*rz_n > ntot/numcalib
            % calculate calibration area
            % r0 -- ratio of num calib to largest dim
            if ny>nz
                r0 = ncal_i/ny;
            else
                r0 = ncal_i/nz;
            end
            % rs(1)*abs(ky(:))<r0 ---> 
            %     if fovz/fovy*ny/nz<1,  
            %                        fovz/fovy*ny/nz*ky < ncal/nmax
            %                                        kz < ncal/nmax
            %     else,                              ky < ncal/nmax
            %                        fovy/fovz*nz/ny*kz < ncal/nmax
            if ccutcal        
                maskcalib = sqrt((rs(1)*ky(:)).^2+(rs(2)*kz(:)).^2)<r0;
            else
                maskcalib = (abs(rs(1)*ky(:))<r0).*(abs(rs(2)*kz(:))<r0);
            end
            numcalib = sum(maskcalib(:));
        
            if ry_n*rz_n>ntot/numcalib
                dbdisp('WARNING! acceleration too great for calib size');
                dbdisp(sprintf('  Adjusting calib size from %d to %d',...
                               ncal_i,ncal_i-1));
                ncal_i = ncal_i-1;
                if ncal_i<ncalmin
                    dbdisp('ERROR! Unable to find sampling pattern');
                    dbdisp(sprintf('  Acc = (%g,%g)',ry_n,rz_n));
                    return;
                end
            end
        end
        
        % tl     = number of rings
        % pspoke = number of spokes for one phase 
        %  (also, number of spokes to fully sample the calib region)
        % if ry_n*rz_n==1
        %    pspoke = floor(ntot/(ry_n*rz_n*tl));
        if fib            
            pspokes = round(floorfib(ntot/(ry_n*rz_n*tl)*calnex)/calnex);
        else
            pspokes = round(ntot/(ry_n*rz_n*tl));
        end
        lcalib = ceil(numcalib/pspokes);

        if lcalib >= tl
            ncal_n = ncal_n - 1;
            calnex_iter = 0;
            if (ncal_n<ncalmin)
                dbdisp(sprintf(['Given calnex of %g, cannot find ' ...
                                'suitable sampling (%d)'],...
                               calnex,ncal_n));
                return;
            end
        else

            % nviews[i] = num of views for i-th ring
            %
            % Two ways to calculate variable-density acceleration:
            %  (1) Smoothly from origin to out (ignore calibration)
            %  (2) Smoothly from calibration region to out
            % ... first way better distributes the samples evenly.        
            r_ind = ((lcalib+1):lring)-1; % (1)
            %r_ind = 1:(lring-lcalib); % (2)        
        
            % Find variability (Rg)
            if ry_n*rz_n <= 1 || isempty(r_ind)
                rg = 0;
            else
                rg = (ntot-pspokes*lring)/sum(r_ind.^vdrate);
            end
            
            % Compute num of views per ring (and round to int)
            nviews = [pspokes*ones(1,lcalib) ...
                      pspokes+rg*(r_ind).^vdrate];
            nviews = round(nviews);
            % Adj last ring to make sure we get the right num of points
            nviews(end) = ntot-sum(nviews(1:(end-1)));
        
            if DBPLOT
                sfigure(DBPLOT+1),plot(nviews),drawnow;
            end

            if calnex>1
                % Estimate acceleration (with calnex)
                act_acc_n = estacc(nviews, pspokes*calnex);
                est_pspokes1ph = pspokes*calnex;
            
                % Adjust acceleration
                sc_tmp = (ry*rz)/act_acc_n;
                % Acceleration should be >= desired...
                %     We can fix acceleration later!
                if sc_tmp <= 1 
                    est_pspokes1ph = ceil(est_pspokes1ph/sc_tmp);
                    break;
                end
                ry_n = sqrt(sc_tmp)*ry_n;
                rz_n = sqrt(sc_tmp)*rz_n;
            
                calnex_iter = calnex_iter+1;
                if calnex_iter > 5
                    % Maybe we were asking for too much!
                    %  -> back off the calibration size
                    calnex_iter = 0;
                    ncal_n = ncal_n - 1;
                              
                    if (ncal_n<ncalmin)
                        dbdisp(sprintf(['Given calnex of %g, cannot find ' ...
                                    'suitable sampling (%d)'],...
                                       calnex,ncal_n));
                        return;
                    end
                end
            else
                est_pspokes1ph = ceil(ntot/lring/ry/rz);
                act_acc_n = ntot/sum(lring*pspokes);
                break;
            end
        end
    end           
    
    if DBPRINT
        dbdisp('---------------------------------------');
        dbdisp('VIEW ORDERING: ACCELERATED RADIAL');
        dbdisp('---------------------------------------');
        dbdisp(sprintf('  Num views:             %d',ntot));
        dbdisp(sprintf('  Num rings:             %d',lring));
        dbdisp(sprintf('  Desired num calib pts: %d',numcalib));
        dbdisp(sprintf('  Actual num calib pts:  %d',lcalib*pspokes));
        dbdisp(sprintf('  Estimated num spokes:  %d',est_pspokes1ph));
        dbdisp(sprintf('  Desired ncal:          %d',ncal));
        dbdisp(sprintf('  Actual ncal:           %d',ncal_n));
        dbdisp(sprintf('  Num spokes for calib: ~%d',pspokes));
        dbdisp(sprintf('  NEX for calib:         %g',calnex));
        dbdisp(sprintf('  Num fib rings:         %d',fib));
        dbdisp(sprintf('  Spoke length:          %d',tl));
        dbdisp(sprintf('  Mode:                  %d',mode));
        dbdisp(sprintf('  Spiral twist:          %d',spiral));
        dbdisp(sprintf('  Corner cut?            %d',cornercut));
        dbdisp(sprintf('  Acceleration tol:      %g',acctol));
        dbdisp(sprintf('  Acc grad factor:       %g',rg));       
        dbdisp(sprintf('  VD rate:               %g',vdrate));
        dbdisp('');
        dbdisp('');        
    end
    if DBPLOT || DEMO
        ringmask = zeros(nsize);
    end
    
    % First find calibration area...
    tmp = v(vact(find(1-maskcalib),end),:);
    tmp = sortrows(tmp,2); % Sort by radius
    vc = v([vact(find(maskcalib),end); ...
            tmp(1:(pspokes*lcalib-numcalib),end)],:);
    vo = v(tmp((1+(pspokes*lcalib-numcalib)):end,end),:); 
    vo = sortrows(vo,2);
    
    clear tmp;

    % Build ring table
    Rtab = []; 
    for R=1:lring
        if R<=lcalib
            indr = (1:nviews(R))+sum(nviews(1:(R-1)));
            %indr = (1:pspokes)+pspokes*(R-1);
            vi = vc(indr,:);
        else
            indr = (1:nviews(R))+sum(nviews((lcalib+1):(R-1)));
            vi = vo(indr,:);
        end
        vi = sortrows(vi,1); % sort by adj-angle
        ri = [];
        ri = [yi(vi(:,end)),zi(vi(:,end))];
        Rtab{R} = vi;

        if DBPLOT || DEMO
            if R<=lcalib || (R>lcalib&&mod(R-lcalib-1,2))
                scalei = 0.5;
            else
                scalei = 1;
            end
            for N=1:size(vi,1)
                ringmask(yi(vi(N,end)),zi(vi(N,end))) = scalei;
            end
        end
    end
    
    mask_act = zeros(nsize);
    I_ph = 0;                    % index for current phase 
    rph  = 1;                    % phase acceleration (fractional phases)
    Rang = zeros(lring,1);       % angle for current index for each ring
    Aph  = zeros(ceil(numph),1); % actual acceleration
    Nph  = zeros(ceil(numph),1); % actual num-spokes per phase
    ytab = []; ztab = [];        % tab (y,z)-indeces: (I_spoke,I_ring)
    I_spoke = 1;                 % current spoke

    while I_ph < numph
        for R=1:lring
            % Indeces from table
            vi = Rtab{R};
                    
            % Increment by golden-angle (~137.5 deg)
            [I_vi,R_vi] = inc_golden(Rang(R), ...
                                     nviews(R));
            Rang(R) = R_vi;
                        
            % Add to table
            ytab(I_spoke,R) = yi(vi(I_vi,end));
            ztab(I_spoke,R) = zi(vi(I_vi,end));
        end
        
        % Add spoke to acquisition mask 
        for N=1:tl
            mask_act(ytab(I_spoke,N),ztab(I_spoke,N)) = 1;
        end
        
        % Adjust for fractional phase
        if (numph-I_ph) <= numph-floor(numph)
            rph = numph-floor(numph);
        end
        
        % Check acceleration 
        acci = ntot/sum(mask_act(:));
        Aph(I_ph+1) = acci;
        ndes = ntot/(ry*rz)*rph;
        if abs(sum(mask_act(:)) - ndes)/ndes <= acctol
            mask_act = zeros(nsize);
            Nph(I_ph+1) = I_spoke-sum(Nph(1:(I_ph+1)));
            I_ph = I_ph+1;
        end
        
        I_spoke = I_spoke+1;
    end
    ntotspoke = I_spoke-1;
    
    if DBPRINT
        dbdisp(sprintf('  Total num spokes:      %d',ntotspoke));
        for N=1:length(Aph)
            dbdisp(sprintf('    [Phase %d] R = %0.2f, Spokes = %g',...
                           N, Aph(N), Nph(N)));
        end
    end
    if DBPLOT
        sfigure(DBPLOT+2),imshow(ringmask,[0 1]),drawnow;
    end
    
    % create pout -- array with phase numbers
    if nargout>=3
        pout = [];
        for N=1:numph
            pout = [pout ones(1,Nph(N)*tl)*N];
        end
    end
    
    %% Finalize
    % Sort each train
    ky = rs(1)*(-1+2*(ytab-1)/ny);
    kz = rs(2)*(-1+2*(ztab-1)/nz);
    kr = abs(ky+i*kz);
    yout = []; zout = [];
    for I=1:ntotspoke
        vi = [kr(I,1:tl).',(1:tl).'];
        vi = sortrows(vi,1);
        switch mode
         case 1 % out to center
          yout = [yout, ytab(I,vi(end:-1:1,end))];
          zout = [zout, ztab(I,vi(end:-1:1,end))];
         case 2 % center out, and back
          i1 = vi(1:2:end,end);
          i2 = vi(2:2:end,end);
          i2 = i2(end:-1:1);
          yout = [yout, ytab(I,i1), ytab(I,i2)];
          zout = [zout, ztab(I,i1), ztab(I,i2)];
         otherwise % center to out
          yout = [yout, ytab(I,vi(:,end))];
          zout = [zout, ztab(I,vi(:,end))];
        end
    end
    
    % Construct resulting sampling masks
    mask_ph1 = zeros(nsize); % mask for phase 1
    mask_all = zeros(nsize); % mask for all phases combined
    for N=1:Nph(1)*lring
        mask_ph1(yout(N),zout(N)) = 1;
    end
    for N=1:ntotspoke*lring
        mask_all(yout(N),zout(N)) = 1;
    end
    if DBPRINT
        dbdisp(sprintf('  Combined acc factor:   %0.2f',ntot/sum(mask_all(:))));
    end
    
    if DBPLOT
        sfigure(DBPLOT+3); imshow(mask_ph1,[]);
        sfigure(DBPLOT+4); imshow(mask_all,[]);
        scale = sum(mask_ph1(:));
        sfigure(DBPLOT+5); imshow(abs(ifftnc(mask_ph1)),[0 1e2/scale]);
        drawnow;
    end
    if DEMO
        sampa = zeros(nsize);
        sampp = zeros(nsize);
        for N=1:length(kadj)
            sampa(yi(N),zi(N)) = va(N);
            sampp(yi(N),zi(N)) = vp(N);
        end
        maxa = max(va(:));
        mina = min(va(:));
        sfigure(DEMO);
        
        subplot(231);
        imshow(sampa,[mina+0.1 maxa]),title('radius (k_r)');%,colorbar;        
        subplot(232);
        imshow(sampp,[]),title('angle (k_\theta)');%,colorbar;       
        subplot(233);
        imshow(ringmask,[0 1]),title('Ring division');
        subplot(234);
        imshow(mask_ph1,[0 1]); title('Sampling mask');
        subplot(235); scale = sum(mask_ph1(:));
        imshow(abs(ifftnc(mask_ph1)),[0 1e2/scale]),title('Undersampling PSF');
        drawnow;
    end
    
    if nargout>=4
        mask = mask_ph1;
    end
    
    return;
    
function [Nvec,Ai] = perm_golden(N,A0)
    gr = ((sqrt(5)-1)/2)*(N-1);
    Ai = A0;
    mask = zeros(N,1);
    Nvec = [];
    while sum(mask(:))~=N
        Ni = round(Ai)+1;
        if ~mask(Ni);
            Nvec = [Nvec;Ni];
        end
        Ai = Ai+gr;
        if Ai>(N-1)
            Ai = Ai-(N-1);
        end
    end
    
function [I,A] = inc_golden(Ac,N)
% INC_GOLDEN increment index using golden cut
%   [I,A] = INC_GOLDEN(Ac,N)
%     Ac -- current angle
%     N  -- max index
%     I  -- next index
%     A  -- next angle
    
    % golden ratio
    gr = (sqrt(5)-1)/2;
    % golden angle
    gra = 360*gr;
    % delta angle
    da = 360/N;
    
    % increment angle (unwrap if needed)
    A = Ac+gra;
    while A>360
        A = A-360;
    end
    % find closest index
    I = round(A/da)+1;
    % unwrap if needed
    if I>N
        I = I-N;
    end
    
function a = estacc(nviews,nspokes)
    m = ones(size(nviews))*nspokes;
    ind = find(m>nviews);
    m(ind) = nviews(ind);
    a = sum(nviews(:))/sum(m(:));
    
function p = parseparam(paramcell)
    if mod(size(paramcell),2) ~= 0
        dbdisp('Invalid parameters!');
        p = -1;
        return;
    end
    
    %% default
    p = [];
    p.type      = 0;
    p.mode      = 0;
    p.corner    = 1;
    p.dofib     = 1;
    p.cornercal = 1;
    p.spiral    = 10;
    p.dbprint   = 0;
    p.dbplot    = 0;
    p.demo      = 0;
    p.calnex    = 1;
    p.ncalmin   = 10;
    p.vdrate    = 1;
    p.acctol  = 1e-2;
    
    %% look through user defined params
    for N=1:(size(paramcell,2)/2)
        pname = paramcell{2*N-1};
        parg  = paramcell{2*N};
        switch pname
         case 'type'
          switch parg
           case 'ellipse'
            p.type = 0;
           case 'square'
            p.type = 1;
           case 'triangle'
            p.type = 2;
           otherwise
            dbdisp(sprintf('Invalid type %s, set to default.',parg));
            p.type = 0;
          end
         case 'mode'
          p.mode = parg;
         case 'corner'
          p.corner = parg;
         case 'dofib'
          p.dofib = parg;
         case 'cornercal'
          p.cornercal = parg;
         case 'spiral'
          p.spiral = parg;
         case 'calnex'
          p.calnex = parg;
         case 'ncalmin'
          p.ncalmin = parg;
         case 'vdrate'
          p.vdrate = parg;
         case 'acctol'
          p.acctol = parg;
         case 'dbplot'
          p.dbplot = parg;
         case 'dbprint'
          p.dbprint = parg;
         case 'demo'
          p.demo = parg;
         otherwise
          dbdisp(sprintf('Ignoring invalid parameter %s.',pname));
        end
    end